@@ -976,29 +976,42 @@ void slow_shut_down (int minutes)
 //   fine, else the game is shut down by a call to this function.
 
 {
-    filter_array(users(), #'tell_object,
+    filter(users(), #'tell_object,
       "Game driver shouts: The memory is getting low !\n");
     "obj/shut"->shut(minutes);
 }
 
 //---------------------------------------------------------------------------
-void notify_shutdown ()
+varargs void notify_shutdown (string crash_reason)
 
-// Notify the master about an immediate shutdown.
+// Notify the master about an immediate shutdown. If <crash_reason> is 0,
+// it is a normal shutdown, otherwise it is a crash and <crash_reason>
+// gives a hint at the reason.
 //
-// If the gamedriver shuts down, this is the last function called before
-// the mud shuts down the udp connections and the accepting socket for new
-// players.
 // The function has the opportunity to perform any cleanup operation, like
 // informing the mudwho server that the mud is down. This can not be done
 // when remove_player() is called because the udp connectivity is already
 // gone then.
+//
+// If the gamedriver shuts down normally , this is the last function called
+// before the mud shuts down the udp connections and the accepting socket
+// for new players.
+//
+// If the gamedriver crashes, this is the last function called before the
+// mud attempts to dump core and exit. WARNING: Since the driver is in
+// an unstable state, this function may not be able to run to completion!
+// The following crash reasons are defined:
+//   "Fatal Error": an internal sanity check failed.
 
 {
     if (previous_object() && previous_object() != this_object())
         return;
-    filter_array(users(), #'tell_object,
-      "Game driver shouts: LPmud shutting down immediately.\n");
+    if (!crash_reason)
+        filter(users(), #'tell_object,
+          "Game driver shouts: LPmud shutting down immediately.\n");
+    else
+        filter(users(), #'tell_object,
+          "Game driver shouts: PANIC! "+ crash_reason+"!\n");
     save_wiz_file();
     mudwho_shutdown();
 }
@@ -1180,9 +1193,10 @@ int privilege_violation (string op, mixed who, mixed arg, mixed arg2)
 //                       is shadowed by a 'nomask'-type simul_efun.
 //   rename_object     : The current object <who> renames object <arg>
 //                       to name <arg2>.
-//   send_imp          : Send UDP-data to host <arg>.
+//   send_imp          : Send UDP-data to host arg3 (deprecated).
+//   send_udp          : Send UDP-data to host <arg>.
 //   set_auto_include_string : Set the string automatically included by
-//                       the compiler.
+//                       the compiler (deprecated).
 //   get_extra_wizinfo : Get the additional wiz-list info for wizard <arg>.
 //   set_extra_wizinfo : Set the additional wiz-list info for wizard <arg>.
 //   set_extra_wizinfo_size : Set the size of the additional wizard info
@@ -1391,7 +1405,7 @@ int valid_snoop (object snoopee, object snooper)
     if (!geteuid(previous_object()))
         return 0;
     */
-    if (file_name(previous_object()) == get_simul_efun())
+    if (object_name(previous_object()) == get_simul_efun())
         return 1;
 }
 
@@ -1476,47 +1490,47 @@ mixed valid_read  (string path, string euid, string fun, object caller)
     string user;
 
     switch ( fun ) {
-	case "restore_object": return 1;
-	case "ed_start":
-	    if ( previous_object() && previous_object() != this_player() )
-		return 0;
-	    if (!path) {
-		/* request for file with last error */
-		mixed *error;
+        case "restore_object": return 1;
+        case "ed_start":
+            if ( previous_object() && previous_object() != this_player() )
+                return 0;
+            if (!path) {
+                /* request for file with last error */
+                mixed *error;
 
-		error =
-		  get_error_file(({string})this_player()->query_real_name());
-		if (!error || error[3]) {
-		    write("No error.\n");
-		    return 0;
-		}
-		write(error[0][1..]+" line "+error[1]+": "+error[2]+"\n");
-		return error[0];
-	    }
-	    if (path[0] != '/')
-		path = "/"+path;
-	case "read_file":
-	case "read_bytes":
-	case "file_size":
-	case "get_dir":
-	case "do_rename":
-	    if (caller == this_object()) return 1;
-	case "tail":
-	case "print_file":
-	case "make_path_absolute": /* internal use, see below */
-	    set_this_object(caller);
-	    if( this_player() && query_ip_number(this_player()) ) {
-	        path = (string)this_player()->valid_read(path);
-	        if (!stringp(path)) {
-	            write("Bad file name.\n");
-	            return 0;
-	        }
-	        return path;
-	    }
-	    path = (string)"obj/player"->valid_read(path);
-	    if (stringp(path))
-	        return path;
-	    return 0;
+                error =
+                  get_error_file(({string})this_player()->query_real_name());
+                if (!error || error[3]) {
+                    write("No error.\n");
+                    return 0;
+                }
+                write(error[0][1..]+" line "+error[1]+": "+error[2]+"\n");
+                return ADD_SLASH(error[0]);
+            }
+            if (path[0] != '/')
+                path = "/"+path;
+        case "read_file":
+        case "read_bytes":
+        case "file_size":
+        case "get_dir":
+        case "do_rename":
+            if (caller == this_object()) return 1;
+        case "tail":
+        case "print_file":
+        case "make_path_absolute": /* internal use, see below */
+            set_this_object(caller);
+            if( this_player() && query_ip_number(this_player()) ) {
+                path = (string)this_player()->valid_read(path);
+                if (!stringp(path)) {
+                    write("Bad file name.\n");
+                    return 0;
+                }
+                return ADD_SLASH(path);
+            }
+            path = (string)"obj/player"->valid_read(path);
+            if (stringp(path))
+                return ADD_SLASH(path);
+            return 0;
     }
     /* if a case failed to return a value or the caller function wasn't
      * recognized, we come here.
@@ -1562,38 +1576,44 @@ mixed valid_write (string path, string euid, string fun, object caller)
 {
     string user;
 
+    if (path[0] == '/' && path != "/")
+        path = path[1..];
+
     switch ( fun ) {
     case "objdump":
-        if (path == "/OBJ_DUMP") return path;
+        if (path == "OBJ_DUMP") return path;
         return 0;
 
     case "opcdump":
-        if (path == "/OPC_DUMP") return path;
+        if (path == "OPC_DUMP") return path;
         return 0;
 
     case "save_object":
         if ( user = GETUID(previous_object()) ) {
-            if ( path[0 .. strlen(user)+8] == "/players/" + user
+            if ( path[0 .. strlen(user)+7] == "players/" + user
              &&  sscanf(path, ".%s", user) == 0)
-                return path;
+                return ADD_SLASH(path);
         } else {
-            user = file_name(previous_object());
+            user = efun::object_name(previous_object());
+#ifndef __COMPAT_MODE__
+            user = user[1..];
+#endif
             if ( user[0..3] == "obj/"
              ||  user[0..4] == "room/"
              ||  user[0..3] == "std/"  )
-                return path;
+                return ADD_SLASH(path);
         }
         return 0; /* deny access */
     default:
         return 0; /* deny access */
     case "write_file":
         if (caller == this_object()) return 1;
-        if (path[0..4] == "/log/"
-         && !(   sizeof(regexp(({path[5..34]}), "/"))
-              || path[5] == '.'
-              || strlen(path) > 35
-	    ) ) {
-            return path;
+        if (path[0..3] == "log/"
+         && !(   sizeof(regexp(({path[4..33]}), "/"))
+              || path[4] == '.'
+              || strlen(path) > 34
+            ) ) {
+            return ADD_SLASH(path);
         }
         break;
     case "ed_start":
@@ -1602,13 +1622,13 @@ mixed valid_write (string path, string euid, string fun, object caller)
         break;
     case "rename_from":
     case "rename_to":
-        if ((   file_name(caller) == SIMUL_EFUN_FILE
-             || file_name(caller) == SPARE_SIMUL_EFUN_FILE)
-         && path[0..4] == "/log/"
-         && !(   sizeof(regexp(({path[5..34]}), "/"))
-              || path[5] == '.'
-              || strlen(path) > 35
-	    ) ) {
+        if ((   efun::object_name(caller) == SIMUL_EFUN_FILE
+             || efun::object_name(caller) == SPARE_SIMUL_EFUN_FILE)
+         && path[0..3] == "log/"
+         && !(   sizeof(regexp(({path[4..33]}), "/"))
+              || path[4] == '.'
+              || strlen(path) > 34
+            ) ) {
             return 1;
         }
     case "mkdir":
@@ -1619,17 +1639,18 @@ mixed valid_write (string path, string euid, string fun, object caller)
     }
 
     set_this_object(caller);
-    if( this_player() && query_ip_number(this_player()) ) {
-	path = (string)this_player()->valid_write(path);
-	if (!stringp(path)) {
-	    write("Bad file name.\n");
-	    return 0;
-	}
-	return path;
+    if( this_player() && query_ip_number(this_player()) )
+    {
+        path = (string)this_player()->valid_write(path);
+        if (!stringp(path)) {
+            write("Bad file name.\n");
+            return 0;
+        }
+        return ADD_SLASH(path);
     }
     path = (string)"obj/player"->valid_write(path);
     if (stringp(path))
-	return path;
+        return ADD_SLASH(path);
 
     return 0;
 }
@@ -1945,12 +1966,11 @@ void save_wiz_file()
  */
 
 {
-#ifdef __WIZLIST__
-    rm(__WIZLIST__);
+    rm("/WIZLIST");
     write_file(
-      __WIZLIST__,
+      "/WIZLIST",
       implode(
-        map_array(wizlist_info(),
+        map(wizlist_info(),
           lambda(({'a}),
             ({#'sprintf, "%s %d %d\n",
               ({#'[, 'a, WL_NAME}),
@@ -1961,7 +1981,6 @@ void save_wiz_file()
         ), ""
       )
     );
-#endif
 }
 
 //---------------------------------------------------------------------------
@@ -1975,8 +1994,8 @@ int verify_create_wizard (object ob)
 {
     int dummy;
 
-    if (sscanf(file_name(ob), "room/port_castle#%d", dummy) == 1
-      || sscanf(file_name(ob), "global/port_castle#%d", dummy) == 1)
+    if (sscanf(object_name(ob), "room/port_castle#%d", dummy) == 1
+      || sscanf(object_name(ob), "global/port_castle#%d", dummy) == 1)
 	return 1;
     return 0;
 }
@@ -2017,7 +2036,7 @@ string master_create_wizard(string owner, string domain, object caller)
 		    wizard + "\n");
 	mkdir(wizard);
     }
-    dest = file_name(environment(player));
+    dest = object_name(environment(player));
     def_castle = "#define NAME \"" + owner + "\"\n#define DEST \"" +
 	dest + "\"\n" + read_file("/room/def_castle.c");
     if (file_size(castle) > 0) {
@@ -2036,3 +2055,4 @@ string master_create_wizard(string owner, string domain, object caller)
 }
 
 /****************************************************************************/
+
