@@ -93,7 +93,7 @@
 // void slow_shut_down (int minutes)
 //   Schedule a shutdown for the near future.
 //
-// void notify_shutdown ()
+// void notify_shutdown (void|string crash_reason)
 //   Notify the master about an immediate shutdown.
 //
 //---------------------------------------------------------------------------
@@ -138,7 +138,7 @@
 //     Userids and depending Security
 //
 // string get_bb_uid()
-//   Return the string to be used as backbone-uid.
+//   Return the string to be used as temporary euid by process_string().
 //
 // int valid_seteuid (object obj, string neweuid)
 //   Validate the change of an objects euid by efun seteuid().
@@ -254,7 +254,7 @@ void inaugurate_master (int arg)
 // mudwho or wizlist handling has to be initialized here.
 //
 // Besides that, do whatever you feel you need to do,
-// e.g. set_auto_include_string(), or give the master a decent euid.
+// e.g. set_auto_include(), or give the master a decent euid.
 
 
 //---------------------------------------------------------------------------
@@ -645,17 +645,26 @@ void slow_shut_down (int minutes)
 //   fine, else the game is shut down by a call to this function.
 
 //---------------------------------------------------------------------------
-void notify_shutdown (void)
+varargs void notify_shutdown (string crash_reason)
 
-// Notify the master about an immediate shutdown.
+// Notify the master about an immediate shutdown. If <crash_reason> is 0,
+// it is a normal shutdown, otherwise it is a crash and <crash_reason>
+// gives a hint at the reason.
 //
-// If the gamedriver shuts down, this is the last function called before
-// the mud shuts down the udp connections and the accepting socket for new
-// players.
 // The function has the opportunity to perform any cleanup operation, like
 // informing the mudwho server that the mud is down. This can not be done
 // when remove_player() is called because the udp connectivity is already
 // gone then.
+//
+// If the gamedriver shuts down normally , this is the last function called
+// before the mud shuts down the udp connections and the accepting socket
+// for new players.
+//
+// If the gamedriver crashes, this is the last function called before the
+// mud attempts to dump core and exit. WARNING: Since the driver is in
+// an unstable state, this function may not be able to run to completion!
+// The following crash reasons are defined:
+//   "Fatal Error": an internal sanity check failed.
 
 
 //===========================================================================
@@ -809,7 +818,7 @@ int privilege_violation (string op, mixed who, mixed arg, mixed arg2)
 //   send_imp          : Send UDP-data to host arg3 (deprecated).
 //   send_udp          : Send UDP-data to host <arg>.
 //   set_auto_include_string : Set the string automatically included by
-//                       the compiler.
+//                       the compiler (deprecated).
 //   get_extra_wizinfo : Get the additional wiz-list info for wizard <arg>.
 //   set_extra_wizinfo : Set the additional wiz-list info for wizard <arg>.
 //   set_extra_wizinfo_size : Set the size of the additional wizard info
@@ -818,7 +827,8 @@ int privilege_violation (string op, mixed who, mixed arg, mixed arg2)
 //   limited:          : Execute <arg> with reduced/changed limits.
 //   set_limits        : Set limits to <arg>.
 //   set_this_object   : Set this_object() to <arg>.
-//   shadow_add_action : Add an action to function <arg> from a shadow.
+//   shadow_add_action : Add an action to function <arg4> of object <arg3>
+//                       from the shadow <who> which is shadowing <arg3>.
 //   symbol_variable   : Attempt to create symbol of a hidden variable
 //                       of object <arg> with with index <arg2> in the
 //                       objects variable table.
@@ -945,9 +955,13 @@ int valid_snoop (object snoopee, object snooper)
 //---------------------------------------------------------------------------
 string get_bb_uid()
 
-// Return the string (or 0) to be used as backbone-euid.
-// It is just used by process_string() only if no this_object() is present.
-// If strict-euids, the function must exist and return a string.
+// This method is called when efun process_string() is used without a
+// current object (e.g. from notify_fail method). The current object
+// will be set to the current command giver, and will receive the euid
+// returned from this function.
+// 
+// If strict-euids, this function must exist and return a string.
+// Otherwise the function is optional and/or may return 0.
 
 //---------------------------------------------------------------------------
 int valid_seteuid (object obj, string neweuid)
