@@ -97,6 +97,7 @@ struct command_context_s
     action_t * marker;       /* the marker sentence */
     char * cmd;              /* the full command, a stack buffer */
     char * verb;             /* the shared verb */
+    char * action_verb;      /* the tabled action verb */
     svalue_t errmsg;         /* the error message */
     object_t * errobj;       /* object which set the error message */
 };
@@ -114,7 +115,12 @@ object_t *command_giver;
 
 char *last_verb = NULL;
   /* During a command execution, this is the shared string with the
-   * command verb.
+   * given command verb.
+   */
+
+char *last_action_verb = NULL;
+  /* During a command execution, this is the tabled string with the
+   * command verb as specified in the action definition.
    */
 
 char *last_command = NULL;
@@ -175,6 +181,10 @@ free_action_temporaries (void)
         free_string(last_verb);
     last_verb = NULL;
 
+    if (last_action_verb)
+        free_string(last_action_verb);
+    last_action_verb = NULL;
+
 } /* free_action_temporaries() */
 
 /*-------------------------------------------------------------------------*/
@@ -259,6 +269,7 @@ save_command_context (struct command_context_s * context)
 {
     context->rt.type = COMMAND_CONTEXT;
     context->verb = last_verb;
+    context->action_verb = last_action_verb;
     context->cmd = last_command;
     context->mark_player = marked_command_giver;
     if (marked_command_giver)
@@ -272,6 +283,7 @@ save_command_context (struct command_context_s * context)
 
     command_giver = NULL;
     last_verb = NULL;
+    last_action_verb = NULL;
     last_command = NULL;
     marked_command_giver = NULL;
     command_marker = NULL;
@@ -292,6 +304,9 @@ _restore_command_context (struct command_context_s * context)
     if (last_verb)
         free_string(last_verb);
 
+    if (last_action_verb)
+        free_string(last_action_verb);
+
     if (command_marker)
         free_action_sent(command_marker);
     else if (marked_command_giver && !(O_DESTRUCTED & marked_command_giver->flags))
@@ -299,6 +314,7 @@ _restore_command_context (struct command_context_s * context)
 
     /* Restore the previous context */
     last_verb = context->verb;
+    last_action_verb = context->action_verb;
     last_command = context->cmd;
     command_giver = check_object(context->this_player);
     if (context->this_player)
@@ -355,7 +371,14 @@ remove_action_sent (object_t *ob, object_t *player)
                                  , time_stamp());
             }
 #endif
+#ifdef CHECK_OBJECT_REF
+            if (s == &player->sent)
+                update_object_sent(player, tmp->sent.next);
+            else
+                *s = tmp->sent.next;
+#else
             *s = tmp->sent.next;
+#endif /* CHECK_OBJECT_REF */
             free_action_sent(tmp);
         }
         else
@@ -399,7 +422,14 @@ remove_shadow_action_sent (object_t *ob, object_t *player)
                                  , time_stamp());
             }
 #endif
+#ifdef CHECK_OBJECT_REF
+            if (s == &player->sent)
+                update_object_sent(player, tmp->sent.next);
+            else
+                *s = tmp->sent.next;
+#else
             *s = tmp->sent.next;
+#endif /* CHECK_OBJECT_REF */
             free_action_sent(tmp);
         }
         else
@@ -442,11 +472,25 @@ remove_environment_sent (object_t *player)
                 s = (action_t *)s->sent.next;
                 free_action_sent(tmp);
                 if (!s) {
+#ifdef CHECK_OBJECT_REF
+                if (p == &player->sent)
+                    update_object_sent(player, NULL);
+                else
                     *p = NULL;
+#else
+                    *p = NULL;
+#endif /* CHECK_OBJECT_REF */
                     return;
                 }
             } while (s->ob == ob);
+#ifdef CHECK_OBJECT_REF
+            if (p == &player->sent)
+                update_object_sent(player, (sentence_t *)s);
+            else
+                *p = (sentence_t *)s;
+#else
             *p = (sentence_t *)s;
+#endif /* CHECK_OBJECT_REF */
         }
         else
         {
@@ -621,6 +665,7 @@ special_parse (char *buff)
             }
             add_message("Dumping to /OBJ_DUMP ... ");
             dumpstat("/OBJ_DUMP");
+            dumpstat_dest("/DEST_OBJ_DUMP");
             add_message("done\n");
             if (no_curobj)
             {
@@ -780,8 +825,8 @@ static Bool
 parse_command (char *buff, Bool from_efun)
 
 /* Take the command in <buff> and execute it.
- * The command_giver and marked_command_giver are set, last_verb
- * and last_command may be set (and will be overwritten).
+ * The command_giver and marked_command_giver are set, last_verb,
+ * last_action_verb and last_command may be set (and will be overwritten).
  *
  * The command will be searched in the list of marked_command_giver.
  *
@@ -936,6 +981,10 @@ parse_command (char *buff, Bool from_efun)
          * Now we have found a special sentence!
          */
 
+        if (last_action_verb)
+            free_string(last_action_verb);
+        last_action_verb = ref_string(sa->verb);
+
 #ifdef DEBUG
         if (d_flag > 1)
             debug_message("%s Local command %s on %s\n", time_stamp()
@@ -1297,7 +1346,8 @@ e_add_action (svalue_t *func, svalue_t *cmd, int flag)
     /* And the commandgiver must be in the vicinity */
     if (ob != command_giver
      && ob->super != command_giver
-     && ob->super != command_giver->super
+     && (ob->super == NULL || ob->super != command_giver->super)
+       /* above condition includes the check command_giver->super == NULL */
      && ob != command_giver->super)
         error("add_action from object '%s' that was not present to '%s'.\n"
              , ob->name, command_giver->name);
@@ -1327,6 +1377,7 @@ e_add_action (svalue_t *func, svalue_t *cmd, int flag)
 
     /* Set str to the function, made shared */
     str = func->u.string;
+    func->type = T_NUMBER;
     if ((string_type = func->x.string_type) != STRING_SHARED)
     {
         char *str2;
@@ -1345,6 +1396,7 @@ e_add_action (svalue_t *func, svalue_t *cmd, int flag)
     {
         /* Set str to the command verb, made shared */
         str = cmd->u.string;
+        cmd->type = T_NUMBER;
         if ((string_type = cmd->x.string_type) != STRING_SHARED)
         {
             char *str2;
@@ -1376,6 +1428,7 @@ e_add_action (svalue_t *func, svalue_t *cmd, int flag)
             {
                 if (-flag >= strlen(p->verb))
                 {
+                    free_action_sent(p);
                     error("Bad arg 3 to add_action(): value %ld larger than verb '%s'.\n"
                          , (long)flag, p->verb);
                     /* NOTREACHED */
@@ -1389,6 +1442,7 @@ e_add_action (svalue_t *func, svalue_t *cmd, int flag)
             }
             else
             {
+                free_action_sent(p);
                 error("Bad arg 3 to add_action(): value %ld too big.\n"
                      , (long)flag);
                 /* NOTREACHED */
@@ -1415,7 +1469,11 @@ e_add_action (svalue_t *func, svalue_t *cmd, int flag)
     else
     {
         p->sent.next = command_giver->sent;
+#ifdef CHECK_OBJECT_REF
+        update_object_sent(command_giver, (sentence_t *)p);
+#else
         command_giver->sent = (sentence_t *)p;
+#endif /* CHECK_OBJECT_REF */
     }
 
     return MY_FALSE;
@@ -1535,7 +1593,14 @@ f_remove_action (svalue_t *sp)
     {
         if (s->ob == ob && s->verb == verb)
         {
+#ifdef CHECK_OBJECT_REF
+            if (sentp == &ob->sent)
+                update_object_sent(ob, s->sent.next);
+            else
+                *sentp = s->sent.next;
+#else
             *sentp = s->sent.next;
+#endif /* CHECK_OBJECT_REF */
             free_action_sent(s);
             break;
         }
@@ -1818,6 +1883,8 @@ f_notify_fail (svalue_t *sp)
             free_object(error_obj, "notify_fail");
         error_obj = ref_object(current_object, "notify_fail");
     }
+    else
+        free_svalue(sp);
 
     put_number(sp, 0);
 
