@@ -144,6 +144,11 @@ Bool garbage_collect_to_do = MY_FALSE;
  * TODO:: struct.
  */
 
+Bool mud_is_up = MY_FALSE;
+  /* True: the driver is finished with the initial processing
+   * and has entered the main loop.
+   */
+
 static double load_av = 0.0;
   /* The load average (player commands/second), weighted over the
    * last period of time.
@@ -329,7 +334,11 @@ cleanup_stuff (void)
     free_old_driver_hooks();
 #endif
 
-    /* Remove all destructed objects.
+    /* Finish up all newly destructed objects.
+     */
+    handle_newly_destructed_objects();
+
+    /* Remove all unreferenced destructed objects.
      */
     remove_destructed_objects();
 
@@ -345,6 +354,11 @@ backend (void)
 
 {
     char buff[MAX_TEXT+4];
+        /* Note that the size of buff[] is determined by MAX_TEXT, which
+         * is the max size of the network receive buffer. Iow: no
+         * buffer overruns possible.
+         */
+
 
 
     /*
@@ -367,6 +381,8 @@ backend (void)
     atexit(exit_alarm_timer);
 #endif
 
+    mud_is_up = MY_TRUE;
+
     printf("%s LDMud ready for users.\n", time_stamp());
     fflush(stdout);
     debug_message("%s LDMud ready for users.\n", time_stamp());
@@ -511,11 +527,6 @@ backend (void)
              */
             (void)time_stamp();
 
-            /* Note that the size of buff[] is determined by MAX_TEXT, which
-             * is the max size of the network receive buffer. Iow: no
-             * buffer overruns possible.
-             */
-
             total_player_commands++;
             update_load_av();
 
@@ -687,6 +698,9 @@ void check_alarm (void)
     static mp_int last_alarm_time = 0;
     mp_int curtime = get_current_time();
 
+    if (t_flag)  /* Timing turned off? */
+        return;
+
     if (!last_alarm_time) /* initialize it */
         last_alarm_time = curtime;
 
@@ -869,6 +883,7 @@ static Bool did_swap;
                 RESET_LIMITS;
                 CLEAR_EVAL_COST;
                 command_giver = 0;
+                previous_ob = NULL;
                 trace_level = 0;
                 reset_object(obj, H_RESET);
                 if (obj->flags & O_DESTRUCTED)
@@ -932,14 +947,17 @@ static Bool did_swap;
                 push_number(0);
             else if (O_PROG_SWAPPED(obj))
                 push_number(1);
+#ifndef NO_BLUEPRINT
             else if (obj->prog->blueprint == obj)
                 push_number(obj->prog->ref - 1);
+#endif /* !NO_BLUEPRINT */
             else
                 push_number(obj->prog->ref);
 
             RESET_LIMITS;
             CLEAR_EVAL_COST;
             command_giver = NULL;
+            previous_ob = NULL;
             trace_level = 0;
             if (driver_hook[H_CLEAN_UP].type == T_CLOSURE)
             {
@@ -1678,6 +1696,7 @@ e_file_size (char *file)
 {
     struct stat st;
 
+    st.st_mode = 0; /* Silences ZeroFault/AIX under high optimizations */
     file = check_valid_path(file, current_object, "file_size", MY_FALSE);
     if (!file)
         return -1;
@@ -1700,7 +1719,8 @@ f_regreplace (svalue_t *sp)
  * Search through <txt> for one/all occurences of <pattern> and replace them
  * with the <replace> pattern, returning the result.
  * <replace> can be a string, or a closure returning a string. If it is
- * a closure, it will be called with the matched substring as argument.
+ * a closure, it will be called with the matched substring and
+ * the position at which it was found as arguments.
  * <flags> is the bit-or of these values:
  *   F_GLOBAL   = 1: when given, all occurences of <pattern> are replace,
  *                   else just the first
@@ -1831,7 +1851,8 @@ f_regreplace (svalue_t *sp)
                 match[patsize] = '\0';
 
                 push_volatile_string(match);
-                call_lambda(subclosure, 1);
+                push_number(pat->startp[0] - start);
+                call_lambda(subclosure, 2);
                 transfer_svalue(&apply_return_value, inter_sp);
                 inter_sp--;
 
