@@ -576,7 +576,7 @@ found:
 
 /*-------------------------------------------------------------------------*/
 size_t
-print_call_out_usage (strbuf_t *sbuf, Bool verbose)
+call_out_status (strbuf_t *sbuf, Bool verbose)
 
 /* Compute and return the amount of memory used by callouts.
  * If <verbose> is true, write detailed statistics to the current user.
@@ -594,7 +594,7 @@ print_call_out_usage (strbuf_t *sbuf, Bool verbose)
     }
     else
     {
-        strbuf_addf(sbuf, "call out:\t\t\t%8ld %8ld (current length %ld)\n"
+        strbuf_addf(sbuf, "call out:\t\t\t%8ld %9ld (current length %ld)\n"
                    , num_call, num_call * sizeof (struct call), num_callouts);
     }
 
@@ -651,7 +651,8 @@ count_extra_ref_from_call_outs (void)
 void
 remove_stale_call_outs (void)
 
-/* GC Support: Remove all callouts referencing destructed objects.
+/* GC and statistics support: Remove all callouts referencing destructed
+ * objects.
  */
 
 {
@@ -694,12 +695,8 @@ clear_ref_from_call_outs (void)
 
         clear_ref_in_callback(&(cop->fun));
 
-        if (NULL != (ob = cop->command_giver) && ob->flags & O_DESTRUCTED)
-        {
-            ob->ref = 0;
-            ob->prog->ref = 0;
-            clear_inherit_ref(ob->prog);
-        }
+        if (NULL != (ob = cop->command_giver))
+            clear_object_ref(ob);
     }
 } /* clear_ref_from_call_outs() */
 
@@ -781,8 +778,13 @@ get_all_call_outs (void)
 
         if (cop->fun.is_lambda)
         {
+            if (cop->fun.function.lambda.x.closure_type == CLOSURE_ALIEN_LFUN)
+                put_ref_object( vv->item
+                              , cop->fun.function.lambda.u.lambda->function.alien.ob
+                              , "get_all_call_outs");
+            else
+                put_ref_object(vv->item, ob, "get_all_call_outs");
             assign_svalue_no_free(&vv->item[1], &cop->fun.function.lambda);
-            /* assuming that item[0] was inited to 0 */
         }
         else
         {
