@@ -49,8 +49,9 @@
  *       makes sure that each string is unique.
  *
  *       The last strings are the names of all files included by the programs
- *       source file, stored in reverse order of their appearance (multiply
- *       included files appear several times).
+ *       source file which also generated code, stored in reverse order of
+ *       their appearance (multiply included files appear several times) and
+ *       as they were given in the source.
  *
  *       When a program is swapped, the reference counts to these strings are
  *       not removed so that the string literals stay in memory all the time.
@@ -597,10 +598,11 @@ struct inherit_s
     unsigned short inherit_type;            /* Type of inherit */
 
 #   define INHERIT_TYPE_NORMAL      0x0000  /* Type: Normal inherit */
-#   define INHERIT_TYPE_EXTRA       0x0001  /* Type: Extra inherit added by
+#   define INHERIT_TYPE_VIRTUAL     0x0001  /* Type: Virtual inherit */
+#   define INHERIT_TYPE_EXTRA       0x0002  /* Type: Extra inherit added by
                                              * copy_variables()
                                              */
-#   define INHERIT_TYPE_DUPLICATE   0x0002  /* Flag: Duplicate virtual inherit */
+#   define INHERIT_TYPE_DUPLICATE   0x0004  /* Flag: Duplicate virtual inherit */
     unsigned short inherit_depth;           /* Depth of inherit */
 };
 
@@ -629,10 +631,12 @@ struct program_s
       /* Name of file that defined prog (allocated, no leading '/',
        * but a trailing '.c')
        */
+#ifndef NO_BLUEPRINT
     object_t       *blueprint;
       /* Counted pointer to the (possibly destructed) blueprint object,
        * or NULL if the blueprint has been destructed in a previous cycle.
        */
+#endif /* !NO_BLUEPRINT */
     int32           id_number;
       /* The id-number is unique among all programs and used to store
        * information for this program without actually pointing to
@@ -673,7 +677,7 @@ struct program_s
        * variables.
        */
     inherit_t *inherit;
-      /* Array [.num_inherited] of descriptors for inherited programs.
+      /* Array [.num_inherited] of descriptors for (directly) inherited programs.
        */
     unsigned short flags;
       /* Flags for the program: */
@@ -722,11 +726,13 @@ struct program_s
     unsigned short num_functions;
       /* Number of functions (inherited and own) of this program */
     unsigned short num_strings;
-      /* Number of shared strings used by the program */
+      /* Number of shared strings (including filenames) used by the program */
+    unsigned short num_includes;
+      /* Number of stored include filenames */
     unsigned short num_variables;
       /* Number of variables (inherited and own) of this program */
     unsigned short num_inherited;
-      /* Number of inherited programs */
+      /* Number of (directly) inherited programs */
 };
 
 
