@@ -36,6 +36,9 @@
  *     void clear_inherit_ref(program_t *p)
  *         Clear the refcounts of all inherited programs of <p>.
  *
+ *     void clear_object_ref(object_t *p)
+ *         Make sure that the refcounts in object <p> are cleared.
+ *
  *     void mark_program_ref(program_t *p);
  *         Set the marker of program <p> and of all data referenced by <p>.
  *
@@ -181,6 +184,8 @@ static size_t num_alloc_strings;
 
 #if !defined(CHECK_STRINGS) && !defined(DEBUG)
 
+#ifndef DUMP_GC_REFS
+
 #define MARK_STRING_REF(str) ((void)(\
     STRING_REFS(str)++ || (\
         CHECK_REF( (str)-sizeof(StrRefCount)-sizeof(char *) ) ||\
@@ -194,6 +199,20 @@ static size_t num_alloc_strings;
    * will return TRUE, otherwise we have an overflow and the STRING_REFS--
    * will undo the ++ from earlier.
    */
+
+#else
+
+#define MARK_STRING_REF(p) \
+    do { char *p_ = p; \
+         dprintf4(gcollect_outfd, "Mark string ref %x (%x): %s %d\n", (long)p_, (long)(p_-sizeof(StrRefCount)-sizeof(char*)), (long)__FILE__, (long)__LINE__); \
+        STRING_REFS(p_)++ \
+        || (CHECK_REF( (p_)-sizeof(StrRefCount)-sizeof(char *) ) \
+            ||  STRING_REFS(p_)-- \
+           ); \
+    } while(0)
+
+#endif /* DUMP_GC_REFS */
+
 #else
 
 static void MARK_STRING_REF (char * str)
@@ -229,7 +248,10 @@ static void MARK_STRING_REF (char * str)
 
 static void clear_map_ref_filter (svalue_t *, svalue_t *, void *);
 static void clear_ref_in_closure (lambda_t *l, ph_int type);
-static void count_ref_in_closure (svalue_t *csvp);
+static void gc_count_ref_in_closure (svalue_t *csvp);
+
+#define count_ref_in_closure(p) \
+    GC_REF_DUMP(svalue_t*, p, "Count ref in closure", gc_count_ref_in_closure)
 
 #endif /* MALLOC_smalloc */
 
@@ -279,7 +301,7 @@ clear_memory_reference (void *p)
 
 /*-------------------------------------------------------------------------*/
 static INLINE void
-note_ref (void *p)
+gc_note_ref (void *p)
 
 /* Note the reference to memory block <p>.
  *
@@ -295,9 +317,11 @@ note_ref (void *p)
         MARK_REF(p);
         return;
     }
-} /* note_ref() */
+} /* gc_note_ref() */
 
-void note_malloced_block_ref (void *p) { note_ref(p); }
+void gc_note_malloced_block_ref (void *p) { gc_note_ref(p); }
+
+#define note_ref(p) GC_REF_DUMP(void*, p, "Note ref", gc_note_ref)
 
 /*-------------------------------------------------------------------------*/
 void
@@ -323,11 +347,44 @@ clear_inherit_ref (program_t *p)
             clear_inherit_ref(p2);
         }
     }
-}
+} /* clear_inherit_ref() */
 
 /*-------------------------------------------------------------------------*/
 void
-mark_program_ref (program_t *p)
+clear_object_ref (object_t *p)
+
+/* If <p> is a destructed object, its refcounts are cleared.
+ * If <p> is a live object, its refcounts are assumed to be cleared
+ * by the GC main method.
+ */
+
+{
+    if ((p->flags & O_DESTRUCTED) && p->ref)
+    {
+#if defined(CHECK_OBJECT_REF) && defined(DEBUG)
+        p->extra_ref = p->ref;
+#endif
+        p->ref = 0;
+        p->prog->ref = 0;
+#ifndef NO_BLUEPRINT
+        if (p->prog->blueprint
+         && (p->prog->blueprint->flags & O_DESTRUCTED)
+         && p->prog->blueprint->ref
+           )
+        {
+#if defined(CHECK_OBJECT_REF) && defined(DEBUG)
+            p->prog->blueprint->extra_ref = p->prog->blueprint->ref;
+#endif
+            p->prog->blueprint->ref = 0;
+        }
+#endif /* !NO_BLUEPRINT */
+        clear_inherit_ref(p->prog);
+    }
+} /* clear_object_ref() */
+
+/*-------------------------------------------------------------------------*/
+void
+gc_mark_program_ref (program_t *p)
 
 /* Set the marker of program <p> and of all data referenced by <p>.
  */
@@ -343,9 +400,15 @@ mark_program_ref (program_t *p)
         variable_t *variable_names;
 
         if (p->ref++)
-            fatal("First reference to program, but ref count != 0\n");
+        {
+            dump_malloc_trace(1, p);
+            fatal("First reference to program %p '%s', but ref count %ld != 0\n"
+                 , p, p->name, (long)p->ref - 1
+                 );
+        }
 
         /* Mark the blueprint object, if any */
+#ifndef NO_BLUEPRINT
         if (p->blueprint)
         {
             if (p->blueprint->flags & O_DESTRUCTED)
@@ -361,6 +424,7 @@ mark_program_ref (program_t *p)
                  */
             }
         }
+#endif /* !NO_BLUEPRINT */
 
         if (p->swap_num != -1 && p->line_numbers)
             note_ref(p->line_numbers);
@@ -407,13 +471,33 @@ mark_program_ref (program_t *p)
     else
     {
         if (!p->ref++)
-            fatal("Program block referenced as something else\n");
+        {
+            dump_malloc_trace(1, p);
+            fatal("Program block %p '%s' referenced as something else\n"
+                 , p, p->name);
+        }
     }
-} /* mark_program_ref() */
+} /* gc_mark_program_ref() */
+
+/*-------------------------------------------------------------------------*/
+static void
+mark_object_ref (object_t *ob)
+
+/* Mark the object <ob> as referenced and increase its refcount.
+ * This method should be called only for destructed objects and
+ * from the GC main loop for the initial count of live objects.
+ */
+
+{
+    MARK_REF(ob); ob->ref++;
+    mark_program_ref(ob->prog);
+    note_ref(ob->name);
+    MARK_STRING_REF(ob->load_name);
+} /* mark_object_ref() */
 
 /*-------------------------------------------------------------------------*/
 void
-reference_destructed_object (object_t *ob)
+gc_reference_destructed_object (object_t *ob)
 
 /* Note the reference to a destructed object <ob>. The referee has to
  * replace its reference by a svalue.number 0 since all these objects
@@ -424,37 +508,41 @@ reference_destructed_object (object_t *ob)
     if (TEST_REF(ob))
     {
         if (ob->ref)
-            fatal("First reference to destructed object, but ref count != 0\n");
+        {
+            dump_malloc_trace(1, ob);
+            fatal("First reference to destructed object %p '%s', "
+                  "but ref count %ld != 0\n"
+                 , ob, ob->name, (long)ob->ref
+                 );
+        }
 
         /* Destructed objects are not swapped */
         ob->next_all = gc_obj_list_destructed;
         gc_obj_list_destructed = ob;
-        MARK_REF(ob);
-        ob->ref++;
-        mark_program_ref(ob->prog);
-        note_ref(ob->name);
-        MARK_STRING_REF(ob->load_name);
+        mark_object_ref(ob);
     }
     else
     {
         if (!ob->ref)
         {
             write_malloc_trace(ob);
-            fatal("Destructed object referenced as something else\n");
+            dump_malloc_trace(1, ob);
+            fatal("Destructed object %p '%s' referenced as something else\n"
+                 , ob, ob->name);
         }
     }
-}
+} /* gc_reference_destructed_object() */
 
 /*-------------------------------------------------------------------------*/
 void
-count_ref_from_string (char *p)
+gc_count_ref_from_string (char *p)
 
 /* Count the reference to shared string <p>.
  */
 
 {
    MARK_STRING_REF(p);
-}
+} /* gc_count_ref_from_string() */
 
 /*-------------------------------------------------------------------------*/
 static void
@@ -491,14 +579,7 @@ clear_ref_in_vector (svalue_t *svp, size_t num)
              * cleared by the obj_list because it is no longer a member
              * Alas, swapped objects must not have prog->ref cleared.
              */
-            if (p->u.ob->flags & O_DESTRUCTED && p->u.ob->ref)
-            {
-                p->u.ob->ref = 0;
-                p->u.ob->prog->ref = 0;
-                if (p->u.ob->prog->blueprint)
-                    p->u.ob->prog->blueprint->ref = 0;
-                clear_inherit_ref(p->u.ob->prog);
-            }
+            clear_object_ref(p->u.ob);
             continue;
 
         case T_POINTER:
@@ -534,14 +615,8 @@ clear_ref_in_vector (svalue_t *svp, size_t num)
                     clear_ref_in_closure(l, p->x.closure_type);
                 }
             }
-            else if (p->u.ob->flags & O_DESTRUCTED && p->u.ob->ref)
-            {
-                p->u.ob->ref = 0;
-                p->u.ob->prog->ref = 0;
-                if (p->u.ob->prog->blueprint)
-                    p->u.ob->prog->blueprint->ref = 0;
-                clear_inherit_ref(p->u.ob->prog);
-            }
+            else
+                clear_object_ref(p->u.ob);
             continue;
         }
     }
@@ -549,9 +624,15 @@ clear_ref_in_vector (svalue_t *svp, size_t num)
 
 /*-------------------------------------------------------------------------*/
 void
-count_ref_in_vector (svalue_t *svp, size_t num)
+gc_count_ref_in_vector (svalue_t *svp, size_t num)
 
 /* Count the references the <num> elements of vector <p>.
+ * Closures bound to destructed objects are replaced by #'undef operator
+ * closures.
+ * TODO: Was the done to preserve sorting orders? Apart from callbacks,
+ * TODO:: replacing it by 0 would do (but see mapping.c). Or even better:
+ * TODO:: introduce an ILLEGAL svalue type for such occasions which compares
+ * TODO:: equal to svalue 0.
  */
 
 {
@@ -641,6 +722,9 @@ count_ref_in_vector (svalue_t *svp, size_t num)
                       p->x.closure_type = F_UNDEF+CLOSURE_EFUN;
                       p->u.ob = master_ob;
                       master_ob->ref++;
+#if defined(CHECK_OBJECT_REF) && defined(DEBUG)
+                      master_ob->extra_ref++;
+#endif
                       reference_destructed_object(ob);
                   }
                   else
@@ -655,7 +739,7 @@ count_ref_in_vector (svalue_t *svp, size_t num)
               continue;
         }
     } /* for */
-}
+} /* gc_count_ref_in_vector() */
 
 /*-------------------------------------------------------------------------*/
 static void
@@ -692,7 +776,7 @@ remove_unreferenced_string (char *start, char *string)
 
 /*-------------------------------------------------------------------------*/
 static void
-note_action_ref (action_t *p)
+gc_note_action_ref (action_t *p)
 
 /* Mark the strings of function and verb of all sentences in list <p>.
  */
@@ -707,13 +791,21 @@ note_action_ref (action_t *p)
     } while ( NULL != (p = (action_t *)p->sent.next) );
 }
 
+#define note_action_ref(p) \
+    GC_REF_DUMP(action_t*, p, "Note action ref", gc_note_action_ref)
+
 /*-------------------------------------------------------------------------*/
 static void
-count_ref_in_closure (svalue_t *csvp)
+gc_count_ref_in_closure (svalue_t *csvp)
 
 /* Count the reference to closure <csvp> and all referenced data.
  * Closures using a destructed object are stored in the stale_ lists
- * for later removal (and .ref is set to -1).
+ * for later removal (and .ref is set to -1), and the svalue is transformed
+ * into a #'undef operator closure.
+ * TODO: Was the transform done to preserve sorting orders? Apart from callbacks,
+ * TODO:: replacing it by 0 would do (but see mapping.c). Or even better:
+ * TODO:: introduce an ILLEGAL svalue type for such occasions which compares
+ * TODO:: equal to svalue 0.
  */
 
 {
@@ -736,6 +828,9 @@ count_ref_in_closure (svalue_t *csvp)
             csvp->x.closure_type = F_UNDEF+CLOSURE_EFUN;
             csvp->u.ob = master_ob;
             master_ob->ref++;
+#if defined(CHECK_OBJECT_REF) && defined(DEBUG)
+            master_ob->extra_ref++;
+#endif
         }
         return;
     }
@@ -783,6 +878,9 @@ count_ref_in_closure (svalue_t *csvp)
                 csvp->x.closure_type = F_UNDEF+CLOSURE_EFUN;
                 csvp->u.ob = master_ob;
                 master_ob->ref++;
+#if defined(CHECK_OBJECT_REF) && defined(DEBUG)
+                master_ob->extra_ref++;
+#endif
             }
         }
         else
@@ -857,26 +955,11 @@ clear_ref_in_closure (lambda_t *l, ph_int type)
         }
     }
 
-    if (type != CLOSURE_UNBOUND_LAMBDA && l->ob->flags & O_DESTRUCTED
-     && l->ob->ref /* block against bad efficency due to multiple refs */ )
-    {
-        l->ob->ref = 0;
-        l->ob->prog->ref = 0;
-        if (l->ob->prog->blueprint)
-            l->ob->prog->blueprint->ref = 0;
-        clear_inherit_ref(l->ob->prog);
-    }
+    if (type != CLOSURE_UNBOUND_LAMBDA)
+        clear_object_ref(l->ob);
 
-    if (type == CLOSURE_ALIEN_LFUN
-     && l->function.alien.ob->flags & O_DESTRUCTED
-     && l->function.alien.ob->ref)
-    {
-        l->function.alien.ob->ref = 0;
-        l->function.alien.ob->prog->ref = 0;
-        if (l->function.alien.ob->prog->blueprint)
-            l->function.alien.ob->prog->blueprint->ref = 0;
-        clear_inherit_ref(l->function.alien.ob->prog);
-    }
+    if (type == CLOSURE_ALIEN_LFUN)
+        clear_object_ref(l->function.alien.ob);
 } /* clear_ref_in_closure() */
 
 /*-------------------------------------------------------------------------*/
@@ -914,6 +997,12 @@ garbage_collection(void)
     size_alloc_strings = 0;
     num_alloc_strings = 0;
 
+    if (gcollect_outfd != 1 && gcollect_outfd != 2)
+    {
+        dprintf1(gcollect_outfd, "\n%s --- Garbage Collection ---\n"
+                               , (long)time_stamp());
+    }
+
     /* --- Pass 0: dispose of some unnecessary stuff ---
      */
 
@@ -925,9 +1014,10 @@ garbage_collection(void)
     malloc_privilege = MALLOC_SYSTEM;
     if (obj_list_replace)
         replace_programs();
-    remove_destructed_objects();
+    handle_newly_destructed_objects();
     free_interpreter_temporaries();
     free_action_temporaries();
+    remove_stale_player_data();
     remove_stale_call_outs();
     free_defines();
     free_all_local_names();
@@ -944,10 +1034,32 @@ garbage_collection(void)
         free_array(current_error_trace);
         current_error_trace = NULL;
     }
+    if (uncaught_error_trace)
+    {
+        free_array(uncaught_error_trace);
+        uncaught_error_trace = NULL;
+    }
+
+    remove_destructed_objects(); /* After reducing all object references! */
+    num_destructed = 0; /* The destructed_objs pointer will be cleared below */
 
     clear_array_size();
     clear_mapping_size();
 
+#ifdef CHECK_OBJECT_REF
+    while (newly_destructed_obj_shadows != NULL)
+    {
+        object_shadow_t *sh = newly_destructed_obj_shadows;
+        newly_destructed_obj_shadows = sh->next;
+        xfree(sh);
+    }
+    while (destructed_obj_shadows != NULL)
+    {
+        object_shadow_t *sh = destructed_obj_shadows;
+        destructed_obj_shadows = sh->next;
+        xfree(sh);
+    }
+#endif /* CHECK_OBJECT_REF */
     /* --- Pass 1: clear the M_REF flag in all malloced blocks ---
      */
     clear_M_REF_flags();
@@ -972,6 +1084,9 @@ garbage_collection(void)
                          , time_stamp(), ob->name);
         }
         was_swapped = 0;
+#if defined(CHECK_OBJECT_REF) && defined(DEBUG)
+        ob->extra_ref = ob->ref;
+#endif
         if (ob->flags & O_SWAPPED
          && (was_swapped = load_ob_from_swap(ob)) & 1)
         {
@@ -980,7 +1095,7 @@ garbage_collection(void)
         else
         {
             /* Take special care of inherited programs, the associated
-             * objects might me destructed.
+             * objects might be destructed.
              */
             ob->prog->ref = 0;
         }
@@ -1001,7 +1116,9 @@ garbage_collection(void)
             } /* end of ed-buffer processing */
         }
         if (was_swapped)
+        {
             swap(ob, was_swapped);
+        }
     }
     if (d_flag > 3)
     {
@@ -1022,29 +1139,13 @@ garbage_collection(void)
         {
             clear_memory_reference(it);
             clear_ref_in_callback(&(it->fun));
+            clear_ref_in_vector(&(it->prompt), 1);
         }
         clear_ref_in_vector(&all_players[i]->prompt, 1);
 
-        if ( NULL != (ob = all_players[i]->snoop_by) )
-        {
-            if (!O_IS_INTERACTIVE(ob))
-            {
-                /* snooping monster */
-                if (ob->flags & O_DESTRUCTED && ob->ref) {
-                    ob->ref = 0;
-                    ob->prog->ref = 0;
-                    clear_inherit_ref(ob->prog);
-                }
-            }
-        } /* end of snoop-processing */
-
-        if ( NULL != (ob = all_players[i]->modify_command) ) {
-            if (ob->flags & O_DESTRUCTED && ob->ref) {
-                ob->ref = 0;
-                ob->prog->ref = 0;
-                clear_inherit_ref(ob->prog);
-            }
-        }
+        /* snoop_by and modify_command are known to be NULL or non-destructed
+         * objects.
+         */
     }
 
     /* Process the driver hooks */
@@ -1079,6 +1180,23 @@ garbage_collection(void)
 
     null_vector.ref = 0;
 
+    /* Finally, walk the of destructed objects and clear all references
+     * in them.
+     */
+    for (ob = destructed_objs; ob;  ob = ob->next_all)
+    {
+        if (d_flag > 4)
+        {
+            debug_message("%s clearing refs for destructed object '%s'\n"
+                         , time_stamp(), ob->name);
+        }
+
+        ob->prog->ref = 0;
+        clear_inherit_ref(ob->prog);
+        ob->ref = 0;
+    }
+
+
     /* --- Pass 3: Compute the ref counts, and set M_REF where appropriate ---
      */
 
@@ -1101,20 +1219,21 @@ garbage_collection(void)
             was_swapped = load_ob_from_swap(ob);
             if (was_swapped & 1)
             {
+#ifdef DUMP_GC_REFS
+                dprintf1(gcollect_outfd, "Clear ref of swapped-in program %x\n", (long)ob->prog);
+#endif
                 CLEAR_REF(ob->prog);
                 ob->prog->ref = 0;
             }
         }
-        ob->ref++;
-        note_ref(ob);
+
+        mark_object_ref(ob);
 
         if (ob->prog->num_variables)
         {
             note_ref(ob->variables);
         }
 
-        mark_program_ref(ob->prog);
-
         count_ref_in_vector(ob->variables, ob->prog->num_variables);
 
         if (ob->sent)
@@ -1142,9 +1261,6 @@ garbage_collection(void)
                 note_action_ref((action_t *)sent);
         }
 
-        note_ref(ob->name);
-        MARK_STRING_REF(ob->load_name);
-
         if (was_swapped)
         {
             swap(ob, was_swapped);
@@ -1167,44 +1283,29 @@ garbage_collection(void)
 
         note_ref(all_players[i]);
 
-        /* There are no destructed interactives */
+        /* There are no destructed interactives, or interactives
+         * referencing destructed objects.
+         */
 
         all_players[i]->ob->ref++;
         if ( NULL != (ob = all_players[i]->snoop_by) )
         {
             if (!O_IS_INTERACTIVE(ob))
             {
-                /* snooping monster */
-                if (ob->flags & O_DESTRUCTED) {
-                    all_players[i]->snoop_by = 0;
-                    reference_destructed_object(ob);
-                } else {
-                    ob->ref++;
-                }
+                ob->ref++;
             }
         } /* end of snoop-processing */
 
         for ( it = all_players[i]->input_to; it != NULL; it = it->next)
         {
-            /* To avoid calling too high-level functions, we want the
-             * input_to_t not to be freed by now.
-             * Thus, we reference the object even if it is destructed.
-             */
             note_ref(it);
             count_ref_in_callback(&(it->fun));
+            count_ref_in_vector(&(it->prompt), 1);
         } /* end of input_to processing */
 
         if ( NULL != (ob = all_players[i]->modify_command) )
         {
-            if (ob->flags & O_DESTRUCTED)
-            {
-                all_players[i]->modify_command = 0;
-                reference_destructed_object(ob);
-            }
-            else
-            {
-                ob->ref++;
-            }
+            ob->ref++;
         }
 
         count_ref_in_vector(&all_players[i]->prompt, 1);
@@ -1272,6 +1373,27 @@ garbage_collection(void)
         }
     }
 
+    /* Finally, walk the of destructed objects and check if the above
+     * processing missed any of them.
+     */
+    for (ob = destructed_objs; ob; )
+    {
+        object_t *next = ob->next_all;
+
+        if (TEST_REF(ob))
+        {
+            dprintf1(gcollect_outfd
+                    , "Freeing destructed object '%s'\n"
+                    , (p_int)ob->name
+                    );
+            ob->ref = 0; /* Since it was never reached by the clear pass */
+            reference_destructed_object(ob);
+        }
+
+        ob = next;
+    }
+    destructed_objs = NULL;
+
     garbage_collection_in_progress = 0;
 
     /* --- Pass 4: remove stralloced strings with M_REF cleared ---
@@ -1289,8 +1411,7 @@ garbage_collection(void)
     for (ob = gc_obj_list_destructed; ob; ob = next_ob)
     {
 #ifdef DEBUG
-#define W(s) write(1,s,strlen(s)) /* DEBUG: */
-W("DEBUG: GC frees destructed '"); W(ob->name); W("'\n");
+        dprintf1(gcollect_outfd, "DEBUG: GC frees destructed '%s'\n", (p_int)ob->name);
 #endif
         next_ob = ob->next_all;
         free_object(ob, "garbage collection");
@@ -1346,7 +1467,19 @@ W("DEBUG: GC frees destructed '"); W(ob->name); W("'\n");
     dprintf2(gcollect_outfd, "%s GC freed %d destructed objects.\n"
             , (long)time_stamp(), dobj_count);
 printf("DEBUG: %s GC found %ld malloced strings using %ld bytes.\n", time_stamp(), (long)num_alloc_strings, (long)size_alloc_strings); /* TODO: Remove this line */
-}
+#if defined(CHECK_OBJECT_REF) && defined(DEBUG)
+    for (ob = obj_list; ob; ob = ob->next_all) {
+        if (ob->extra_ref != ob->ref
+         && strchr(ob->name, '#') == NULL
+           )
+        {
+            dprintf4(2, "DEBUG: GC object %x '%s': refs %d, extra_refs %d\n"
+                      , (p_int)ob, (p_int)ob->name, (p_int)ob->ref
+                      , (p_int)ob->extra_ref);
+        }
+    }
+#endif
+} /* garbage_collection() */
 
 
 #if defined(MALLOC_TRACE)
@@ -1678,9 +1811,10 @@ garbage_collection (void)
 
 {
     assert_master_ob_loaded();
-    remove_destructed_objects();
+    handle_newly_destructed_objects();
     free_interpreter_temporaries();
     free_action_temporaries();
+    remove_stale_player_data();
     remove_stale_call_outs();
     free_defines();
     free_all_local_names();
@@ -1694,6 +1828,12 @@ garbage_collection (void)
         free_array(current_error_trace);
         current_error_trace = NULL;
     }
+    if (uncaught_error_trace)
+    {
+        free_array(uncaught_error_trace);
+        uncaught_error_trace = NULL;
+    }
+    remove_destructed_objects();
 
     reallocate_reserved_areas();
     time_last_gc = time(NULL);
