@@ -93,6 +93,8 @@ Bool check_string_table_flag = MY_FALSE;
 
 Bool strict_euids = MY_FALSE;  /* Enforce use of the euids */
 
+static uint32 random_seed = 0;  /* The seed for the pseudo-random generator. */
+
 /* -- Configuration options -- */
 
 long time_to_reset          = TIME_TO_RESET;
@@ -182,11 +184,13 @@ main (int argc, char **argv)
     /* On some systems, SIGALRM is sometimes blocked.
      * The reasons are unknown, but this seems to be the cure.
      */
+    sigemptyset(&set);
     sigaddset(&set, SIGALRM);
     sigprocmask(SIG_UNBLOCK, &set, 0);
 
     /* Initialisations */
 
+    get_stack_direction();
     init_interpret();
 #ifdef RXCACHE_TABLE
     rxcache_init();
@@ -198,7 +202,8 @@ main (int argc, char **argv)
     assoc_shared_string_key.x.string_type = STRING_SHARED;
 
     current_time = get_current_time();
-    seed_random((uint32)current_time);
+    random_seed = (uint32)current_time;
+    seed_random(random_seed);
 
     dummy_current_object_for_loads = NULL_object;
 #ifdef DEBUG
@@ -265,6 +270,11 @@ main (int argc, char **argv)
           );
       /* This also assures the existance of the fd for the debug log */
 
+    printf("%s Random seed: 0x%lx\n"
+          , time_stamp(), (unsigned long)random_seed);
+    debug_message("%s Random seed: 0x%lx\n"
+                 , time_stamp(), (unsigned long)random_seed);
+
     /* If the master_name hasn't been set, select a sensible default */
     if ('\0' == master_name[0])
     {
@@ -426,7 +436,7 @@ main (int argc, char **argv)
     apply_master_ob(STR_SHUTDOWN, 0);
     ipc_remove();
     remove_all_players();
-    remove_destructed_objects();
+    handle_newly_destructed_objects();
       /* Will perform the remove_interactive calls */
     unlink_swap_file();
 #ifdef DEALLOCATE_MEMORY_AT_SHUTDOWN
@@ -606,6 +616,16 @@ writed (int d, p_uint i)
     } while (j > 0);
 }
 
+/*-------------------------------------------------------------------------*/
+void
+writes (int d, const char *s)
+
+/* Memory safe function to string <s> to fd <d>. */
+
+{
+    write(d, s, strlen(s));
+}
+
 /*-------------------------------------------------------------------------*/
 char *
 dprintf_first (int fd, char *s, p_int a)
@@ -686,6 +706,19 @@ dprintf3 (int fd, char *s, p_int a, p_int b, p_int c)
     dprintf2(fd, s, b, c);
 } /* dprintf3() */
 
+/*-------------------------------------------------------------------------*/
+void
+dprintf4 (int fd, char *s, p_int a, p_int b, p_int c, p_int d)
+
+/* Write a message <s> to <fd>. <s> may contain three %-style formatter.
+ * for the arguments <a>, <b>, <c> and <d>.
+ */
+
+{
+    s = dprintf_first(fd, s, a);
+    dprintf3(fd, s, b, c, d);
+} /* dprintf4() */
+
 /*=========================================================================*/
 /*                        The argument parser                              */
 /*=========================================================================*/
@@ -818,6 +851,7 @@ typedef enum OptNumber {
  , cNoHeart         /* --no-heart           */
  , cNoPreload       /* --no-preload         */
  , cPidFile         /* --pidfile            */
+ , cRandomSeed      /* --random-seed        */
  , cResetTime       /* --reset-time         */
  , cReserved        /* -r                   */
  , cReserveUser     /* --reserve-user       */
@@ -906,6 +940,7 @@ static LongOpt aLongOpts[]
     , { "no-heart",           cNoHeart,        MY_FALSE }
     , { "no-preload",         cNoPreload,      MY_FALSE }
     , { "pidfile",            cPidFile,        MY_TRUE }
+    , { "random-seed",        cRandomSeed,     MY_TRUE }
     , { "reset-time",         cResetTime,      MY_TRUE }
     , { "reserve-user",       cReserveUser,    MY_TRUE }
     , { "reserve-master",     cReserveMaster,  MY_TRUE }
@@ -1358,6 +1393,7 @@ shortusage (void)
 #ifdef YYDEBUG
 "  --y|--yydebug\n"
 #endif
+"  --random-seed <num>\n"
 #ifdef DEBUG
 "  --check-refcounts\n"
 "  --check-state <lvl>\n"
@@ -1530,6 +1566,10 @@ usage (void)
 "    Enable debugging of the LPC compiler.\n"
 "\n"
 #endif
+"  --random-seed <num>\n"
+"    Seed value for the random number generator. If not given, the\n"
+"    driver chooses a seed value on its own.\n"
+"\n"
 #ifdef DEBUG
 "  --check-refcounts\n"
 "    Every backend cycle, all refcounts in the system are checked.\n"
@@ -1665,7 +1705,7 @@ eval_arg (int eOption, const char * pValue)
 
             /* Skip leading spaces */
             *begin_arg++ = '\0';
-            while (*begin_arg == ' ') cp++;
+            while (*begin_arg == ' ') begin_arg++;
 
             /* Count the arguments */
             for (num_args = 0, cp = begin_arg; *cp != '\0'; )
@@ -1678,7 +1718,7 @@ eval_arg (int eOption, const char * pValue)
                 while (*cp == ' ') cp++;
             }
 
-            if (num_args != 0);
+            if (num_args != 0)
             {
                 /* There are arguments!
                  * Put them into the argument array.
@@ -1872,6 +1912,11 @@ eval_arg (int eOption, const char * pValue)
         debug_file = strdup(pValue);
         break;
 
+    case cRandomSeed:
+        random_seed = strtoul(pValue, (char **)0, 0);
+        seed_random(random_seed);
+        break;
+
     case cReserved:
     case cReserveUser:
     case cReserveMaster:
