@@ -211,23 +211,38 @@ object_t NULL_object = { 0 };
    */
 
 /*-------------------------------------------------------------------------*/
+#ifndef CHECK_OBJECT_REF
 void
 _free_object (object_t *ob)
+#else
+void
+_free_object (object_t *ob, const char * file, int line)
+#endif
 
 /* Deallocate/dereference all memory and structures held by <ob>.
- * At the time of call, the object must be have no refcounts lefts,
- * must be destructed and removed from the object table and list.
+ * At the time of call, the object must be have at no refcount left,
+ * must be destructed and removed from the object table and lists.
  */
 
 {
 
 #ifdef DEBUG
 
-    /* Decrement and check the reference count */
+    /* Check the reference count */
 
     if (ob->ref > 0)
         fatal("Object with %ld refs passed to _free_object()\n", ob->ref);
 
+#if 0 && defined(CHECK_OBJECT_REF)
+    if (strchr(ob->name, '#') == NULL)
+        printf("DEBUG: (%s:%d) free_object(%p '%s') ref %ld flags %x\n"
+              , file, line, ob, ob->name, ob->ref, ob->flags);
+#elif defined(CHECK_OBJECT_REF)
+#   ifdef __MWERKS__
+#       pragma unused(file)
+#       pragma unused(line)
+#   endif
+#endif
     if (d_flag)
         printf("%s free_object: %s.\n", time_stamp(), ob->name);
 
@@ -241,7 +256,9 @@ _free_object (object_t *ob)
 #endif /* DEBUG */
 
     if (ob->sent)
-        fatal("Tried to free an object with sentences.\n");
+        fatal("free_object: Object '%s' (ref %ld, flags %08x) "
+              "still has sentences.\n"
+             , ob->name, ob->ref, ob->flags);
 
     /* If the program is freed, then we can also free the variable
      * declarations.
@@ -445,16 +462,21 @@ do_free_sub_strings (int num_strings,   char **strings
 }
 
 /*-------------------------------------------------------------------------*/
+#ifndef CHECK_OBJECT_REF
 void
-free_prog (program_t *progp, Bool free_sub_strings)
+free_prog (program_t *progp, Bool free_all)
+#else
+void
+_free_prog (program_t *progp, Bool free_all, const char * file, int line)
+#endif
 
 /* Decrement the refcount for program <progp>. If it reaches 0, the program
  * is freed.
  *
- * If free_sub_strings is TRUE, all object strings are freed, and
- * free_prog() is called for all inherited programs.
+ * If free_all is TRUE, all object strings and the blueprint reference are
+ * freed, and free_prog() is called for all inherited programs.
  *
- * The only case when free_sub_strings is not true, is, when the swapper
+ * The only case when free_all is not true, is, when the swapper
  * swapped out the program and now attempts to free the memory.
  * This means that the string data is kept in memory all the time.
  * TODO: Swapping the strings is tricky, as they are all shared.
@@ -470,11 +492,36 @@ free_prog (program_t *progp, Bool free_sub_strings)
     if (progp->ref > 0)
         return;
 
+#if 0 && defined(CHECK_OBJECT_REF)
+    if (strchr(progp->name, '#') == NULL)
+        printf("DEBUG: (%s:%d) free_prog(%p '%s') ref %ld\n"
+              , file, line, progp, progp->name, progp->ref);
+#endif
     if (d_flag)
         printf("%s free_prog: %s\n", time_stamp(), progp->name);
     if (progp->ref < 0)
         fatal("Negative ref count for prog ref.\n");
 
+#ifndef NO_BLUEPRINT
+    if (free_all && progp->blueprint)
+    {
+        object_t * blueprint = progp->blueprint;
+        progp->blueprint = NULL;
+#if 0 && defined(CHECK_OBJECT_REF)
+    if (strchr(blueprint->name, '#') == NULL)
+        printf("DEBUG: (%s:%d) free_prog(%p '%s') ref %ld : blueprint (%p '%s') ref %ld, flags %x\n"
+              , file, line, progp, progp->name, progp->ref
+              , blueprint, blueprint->name, blueprint->ref, blueprint->flags);
+#elif defined(CHECK_OBJECT_REF)
+#   ifdef __MWERKS__
+#       pragma unused(file)
+#       pragma unused(line)
+#   endif
+#endif
+        free_object(blueprint, "free_prog");
+    }
+#endif /* !NO_BLUEPRINT */
+
     /* Update the statistics */
     total_prog_block_size -= progp->total_size;
     total_num_prog_blocks -= 1;
@@ -492,7 +539,7 @@ free_prog (program_t *progp, Bool free_sub_strings)
     /* Is it a 'real' free? Then dereference all the
      * things held by the program, too.
      */
-    if (free_sub_strings)
+    if (free_all)
     {
         int i;
         bytecode_p program;
@@ -537,7 +584,7 @@ free_prog (program_t *progp, Bool free_sub_strings)
 
     /* Remove the program structure */
     xfree(progp);
-}
+} /* free_prog() */
 
 /*-------------------------------------------------------------------------*/
 void
@@ -769,14 +816,6 @@ replace_programs (void)
             replace_program_lambda_adjust(r_ob);
         }
 
-        /* Free the old program, finally */
-        free_prog(old_prog, MY_TRUE);
-
-#ifdef DEBUG
-        if (d_flag)
-            debug_message("%s program freed.\n", time_stamp());
-#endif
-
         /* Remove current shadows */
 
         if (r_ob->ob->flags & O_SHADOW)
@@ -809,6 +848,14 @@ replace_programs (void)
             }
         }
         xfree(r_ob);
+
+        /* Free the old program, finally */
+        free_prog(old_prog, MY_TRUE);
+
+#ifdef DEBUG
+        if (d_flag)
+            debug_message("%s program freed.\n", time_stamp());
+#endif
     }
 
     /* Done with the list */
@@ -1414,6 +1461,8 @@ save_svalue (svalue_t *v, char delimiter, Bool writable)
 {
     Bool rc = MY_TRUE;
 
+    assert_stack_gap();
+
     switch(v->type)
     {
     case T_STRING:
@@ -2281,6 +2330,12 @@ restore_mapping (svalue_t *svp, char **str)
             return MY_FALSE;
         }
         data = get_map_lvalue_unchecked(z, &key);
+        if (!data)
+        {
+            outofmemory("restored mapping entry");
+            /* NOTREACHED */
+            return MY_FALSE;
+        }
         free_svalue(&key);
         while (--i >= 0) {
             if (data->type != T_INVALID && data->type != T_NUMBER)
@@ -2487,6 +2542,8 @@ restore_svalue (svalue_t *svp, char **pt, char delimiter)
 {
     char *cp;
 
+    assert_stack_gap();
+
     switch( *(cp = *pt) )
     {
 
@@ -2982,6 +3039,9 @@ f_restore_object (svalue_t *sp)
         space = strchr(cur, ' ');
         if (!file)
             pt = strchr(cur, '\n');
+        else
+            pt = NULL;
+
         if (space == NULL || (!file && pt && pt < space))
         {
             /* No space? It must be the version line! */
