@@ -8,16 +8,16 @@
  */
 
 /* $Format: "#define RELEASE_DATE \"$ProjectDate$\""$ */
-#define RELEASE_DATE "Sun, 14 Oct 2001 21:12:06 -0600"
+#define RELEASE_DATE "Sun, 31 Mar 2002 22:54:39 -0700"
 
 /* $Format: "#define PROJ_VERSION \"$ProjectVersion$\""$ */
-#define PROJ_VERSION "3-2-dev.351"
+#define PROJ_VERSION "3-2-dev.425"
 
 /* $Format: "#define GAME_VERSION \"$ReleaseVersion$\""$ */
 #define GAME_VERSION "3.2.9"
 
 /* $Format: "#define LONG_VERSION \"$ReleaseVersion$-$ReleaseType$.$ProjectMinorVersion$\""$ */
-#define LONG_VERSION "3.2.9-dev.351"
+#define LONG_VERSION "3.2.9-dev.425"
 
 /* $Format: "#define VERSION_MAJOR \"$ReleaseMajor$\""$ */
 #define VERSION_MAJOR "3"
@@ -29,7 +29,7 @@
 #define VERSION_MICRO "9"
 
 /* $Format: "#define VERSION_PATCH \"$ProjectMinorVersion$\""$ */
-#define VERSION_PATCH "351"
+#define VERSION_PATCH "425"
 
 /* $Format: "#define LOCAL_LEVEL \"\""$ */
 #define LOCAL_LEVEL ""
