@@ -10,12 +10,12 @@
  * This version supports the following as modifiers:
  *  " "   pad positive integers with a space.
  *  "+"   pad positive integers with a plus sign.
- *  "-"   left adjusted within field size.
- *        NB: std (s)printf() defaults to right justification, which is
+ *  "-"   left aligned within field size.
+ *        NB: std (s)printf() defaults to right alignment, which is
  *            unnatural in the context of a mainly string based language
  *            but has been retained for "compatability" ;)
  *  "|"   centered within field size.
- *  "$"   flushed to field size. Makes sense only for strings and columns
+ *  "$"   justified to field size. Makes sense only for strings and columns
  *        of strings.
  *  "="   column mode if strings are greater than field size.  this is only
  *        meaningful with strings, all other types are ignored. The strings
@@ -106,11 +106,11 @@
  *                                0100 : string;
  *                                0101 : integer;
  *                                0110 : float
- *   00000000 00xx0000 : justification INFO_J:
+ *   00000000 00xx0000 : aligment INFO_A:
  *                                00 : right;
  *                                01 : centre;
  *                                10 : left;
- *                                11 : flush;
+ *                                11 : justified;
  *   00000000 xx000000 : positive pad char INFO_PP:
  *                                00 : none;
  *                                01 : ' ';
@@ -134,10 +134,10 @@ typedef unsigned int format_info;
 #define INFO_T_INT    0x6
 #define INFO_T_FLOAT  0x7
 
-#define INFO_J        0x30
-#define INFO_J_CENTRE 0x10
-#define INFO_J_LEFT   0x20
-#define INFO_J_FLUSH  0x30
+#define INFO_A         0x30 /* Right alignment */
+#define INFO_A_CENTRE  0x10
+#define INFO_A_LEFT    0x20
+#define INFO_A_JUSTIFY 0x30
 
 #define INFO_PP       0xC0
 #define INFO_PP_SPACE 0x40
@@ -152,7 +152,7 @@ typedef unsigned int format_info;
 
 /*-------------------------------------------------------------------------*/
 
-#define BUFF_SIZE 65536
+#define BUFF_SIZE 0x20000  /* 128 KByte */
   /* Max size of returned string.
    */
 
@@ -222,7 +222,6 @@ struct ColumnSlashTable
     cst            *next;    /* next column structure */
 };
 
-
 /* --- struct sprintf_buffer: dynamic string buffer
  *
  * The structure implements a dynamic string buffer. The structure
@@ -1120,12 +1119,11 @@ svalue_to_string ( fmt_state_t *st
 
 /*-------------------------------------------------------------------------*/
 static void
-add_flush ( fmt_state_t *st
-          , char *str, size_t len, int fs
-          )
+add_justified ( fmt_state_t *st
+              , char *str, size_t len, int fs
+              )
 
-/* Justify string <str> (length <len>) within the fieldsize <fs> to flush
- * at both borders.
+/* Justify string <str> (length <len>) within the fieldsize <fs>.
  * After that, add it to the global buff[].
  */
 
@@ -1172,7 +1170,7 @@ add_flush ( fmt_state_t *st
 
 #ifdef DEBUG
     if (fs < num_words - 1 + num_chars)
-        fatal("add_flush(): fieldsize %d < data length %d\n", fs, num_words - 1 + num_chars);
+        fatal("add_justified(): fieldsize %d < data length %d\n", fs, num_words - 1 + num_chars);
 #endif
 
     /* Compute the number of spaces we need to insert.
@@ -1180,8 +1178,11 @@ add_flush ( fmt_state_t *st
      * at least one space between each word.
      */
     num_spaces = fs - num_chars;
-    min_spaces = num_spaces / (num_words-1);
-      /* min_spaces * (num_words-1) <= num_spaces < min_spaces * num_words */
+    if (num_words == 1)
+        min_spaces = num_spaces;
+    else
+        min_spaces = num_spaces / (num_words-1);
+        /* min_spaces * (num_words-1) <= num_spaces < min_spaces * num_words */
 
     /* Loop again over the data, now adding spaces as we go. */
 
@@ -1221,15 +1222,15 @@ add_flush ( fmt_state_t *st
         /* Find the start of the next word */
         for ( ; pos < len && str[pos] == ' '; pos++) NOOP;
     }
-} /* add_flush() */
+} /* add_justified() */
 
 /*-------------------------------------------------------------------------*/
 static void
-add_justified ( fmt_state_t *st
-              , char *str, size_t len, char *pad, int fs
-              , format_info finfo)
+add_aligned ( fmt_state_t *st
+            , char *str, size_t len, char *pad, int fs
+            , format_info finfo)
 
-/* Justify string <str> (length <len>) within the fieldsize <fs> according
+/* Align string <str> (length <len>) within the fieldsize <fs> according
  * to the <finfo>. After that, add it to the global buff[].
  */
 
@@ -1246,11 +1247,11 @@ add_justified ( fmt_state_t *st
     if (pad[0] == ' ' && pad[1] == '\0' && !(finfo & INFO_PS_KEEP))
         is_space_pad = MY_TRUE;
 
-    switch(finfo & INFO_J)
+    switch(finfo & INFO_A)
     {
-    case INFO_J_FLUSH:
-    case INFO_J_LEFT:
-        /* Also called for the last line of a flush block */
+    case INFO_A_JUSTIFY:
+    case INFO_A_LEFT:
+        /* Also called for the last line of a justified block */
         ADD_STRN(str, len)
         if (is_space_pad)
             sppos = st->bpos;
@@ -1259,7 +1260,7 @@ add_justified ( fmt_state_t *st
             st->sppos = sppos;
         break;
 
-    case INFO_J_CENTRE:
+    case INFO_A_CENTRE:
         if (finfo & INFO_PS_ZERO)
         {
             ADD_PADDING("0", (fs - len + 1) >> 1)
@@ -1277,7 +1278,7 @@ add_justified ( fmt_state_t *st
         break;
 
     default:
-      { /* std (s)printf defaults to right justification.
+      { /* std (s)printf defaults to right alignment.
          */
         if (finfo & INFO_PS_ZERO)
         {
@@ -1290,7 +1291,7 @@ add_justified ( fmt_state_t *st
         ADD_STRN(str, len)
       }
     }
-} /* end of add_justified() */
+} /* add_aligned() */
 
 /*-------------------------------------------------------------------------*/
 static int
@@ -1315,7 +1316,7 @@ add_column (fmt_state_t *st, cst **column)
     /* Set done to the actual number of characters to copy.
      */
     length = COL->pres;
-    if ((COL->info & INFO_J) == INFO_J_FLUSH && length > COL->size)
+    if ((COL->info & INFO_A) == INFO_A_JUSTIFY && length > COL->size)
         length = COL->size;
     for (p = COL_D; length && *p && *p !='\n'; p++, length--) NOOP;
     done = p - COL_D;
@@ -1338,23 +1339,51 @@ add_column (fmt_state_t *st, cst **column)
                 break;
             }
             if (*p == ' ')
-                break;
-        }
-    }
+            {
+                /* If went more than one character back, check if
+                 * the next word is longer than permitted. If that is
+                 * the case we might as well start breaking it up right
+                 * here.
+                 */
+                if (save-2 > done)
+                {
+                    char *p2;
 
-    /* On flush formatting, don't format the last line that way, nor
-     * flush lines ending in NL.
+                    length = COL->pres;
+                    if ((COL->info & INFO_A) == INFO_A_JUSTIFY
+                     && length > COL->size)
+                        length = COL->size;
+                    for ( p2 = p+1, length--
+                        ; length && *p2 && *p2 !='\n' && *p2 != ' '
+                        ; p2++, length--) NOOP;
+                    if (*p2 && *p2 != '\n' && *p2 != ' ')
+                    {
+                        /* Yup, the next word is far too long. */
+                        p += save - done;
+                        done = save - 1;
+                    }
+                    /* else: the next word is not too long */
+                }
+                /* else: breaking too long word here would look silly anyway
+                 */
+                break;
+            } /* if (p == ' ') */
+        } /* for (done) */
+    } /* if (breaking needed) */
+
+    /* On justified formatting, don't format the last line that way, nor
+     * justified lines ending in NL.
      */
-    if ((COL->info & INFO_J) == INFO_J_FLUSH
+    if ((COL->info & INFO_A) == INFO_A_JUSTIFY
      && *COL_D && *(COL_D+1)
      && *p != '\n' && *p != '\0'
        )
     {
-        add_flush(st, COL_D, p - COL_D, COL->size);
+        add_justified(st, COL_D, p - COL_D, COL->size);
     }
     else
     {
-        add_justified(st, COL_D, p - COL_D, COL->pad, COL->size, COL->info);
+        add_aligned(st, COL_D, p - COL_D, COL->pad, COL->size, COL->info);
     }
 
     COL_D += done; /* inc'ed below ... */
@@ -1406,7 +1435,7 @@ add_table (fmt_state_t *st, cst **table)
         /* Get the length to add */
         for (done = 0; (TAB_D[done]) && (TAB_D[done] != '\n'); done++) NOOP;
 
-        add_justified(st, TAB_D, done, TAB->pad, TAB->size, TAB->info);
+        add_aligned(st, TAB_D, done, TAB->pad, TAB->size, TAB->info);
 
         TAB_D += done; /* inc'ed next line ... */
         if (!(*TAB_D) || !(*(++TAB_D)))
@@ -1469,6 +1498,7 @@ static char buff[BUFF_SIZE]; /* The buffer to return the result */
     int           pres;            /* precision */
     unsigned int  err_num;         /* error code */
     char         *pad;             /* fs pad string */
+    int           column_stat;     /* Most recent column add status */
 
 #   define GET_NEXT_ARG {\
         if (++arg >= argc) ERROR(ERR_TO_FEW_ARGS); \
@@ -1520,7 +1550,8 @@ static char buff[BUFF_SIZE]; /* The buffer to return the result */
         while ( NULL != (tcst = st->csts) )
         {
             st->csts = tcst->next;
-            if (tcst->info & INFO_TABLE && tcst->d.tab)
+            if ((tcst->info & (INFO_COLS|INFO_TABLE)) == INFO_TABLE
+             && tcst->d.tab)
                 xfree(tcst->d.tab);
             xfree(tcst);
         }
@@ -1615,6 +1646,7 @@ static char buff[BUFF_SIZE]; /* The buffer to return the result */
 
     format_char = 0;
     nelemno = 0;
+    column_stat = 0;
 
     arg = -1;
     st->bpos = 0;
@@ -1628,12 +1660,16 @@ static char buff[BUFF_SIZE]; /* The buffer to return the result */
         {
             /* Line- or Format end */
 
-            int column_stat = 0;
-
             if (!st->csts)
             {
-                /* No columns/tables to resolve */
+                /* No columns/tables to resolve, but add a second
+                 * newline if there is one pending from an added
+                 * column
+                 */
 
+                if (column_stat == 2)
+                    ADD_CHAR('\n');
+                column_stat = 0;
                 if (!format_str[fpos])
                     break;
                 ADD_CHAR('\n');
@@ -1641,6 +1677,9 @@ static char buff[BUFF_SIZE]; /* The buffer to return the result */
                 continue;
             }
 
+            column_stat = 0; /* If there was a newline pending, it
+                              * will be implicitely added now.
+                              */
             ADD_CHAR('\n');
             st->line_start = st->bpos;
 
@@ -1680,7 +1719,7 @@ static char buff[BUFF_SIZE]; /* The buffer to return the result */
                 st->line_start = st->bpos;
             } /* while (csts) */
 
-            if (column_stat == 2)
+            if (column_stat == 2 && format_str[fpos] != '\n')
                 ADD_CHAR('\n');
 
             if (!format_str[fpos])
@@ -1762,8 +1801,11 @@ static char buff[BUFF_SIZE]; /* The buffer to return the result */
                                     ERROR(ERR_INVALID_STAR);
                                 if ((int)(fs = carg->u.number) < 0)
                                 {
-                                    fs = -fs;
-                                    finfo |= INFO_J_LEFT;
+                                    if (fs == PINT_MIN)
+                                        fs = PINT_MAX;
+                                    else
+                                        fs = -fs;
+                                    finfo |= INFO_A_LEFT;
                                 }
 
                                 if (pres == -2)
@@ -1787,7 +1829,7 @@ static char buff[BUFF_SIZE]; /* The buffer to return the result */
 
                     fpos--; /* bout to get incremented */
                     continue;
-                } /* if (precision/justification field) */
+                } /* if (precision/alignment field) */
 
                 /* fpos now points to format type */
 
@@ -1795,9 +1837,9 @@ static char buff[BUFF_SIZE]; /* The buffer to return the result */
                 {
                 case ' ': finfo |= INFO_PP_SPACE; break;
                 case '+': finfo |= INFO_PP_PLUS; break;
-                case '-': finfo |= INFO_J_LEFT; break;
-                case '|': finfo |= INFO_J_CENTRE; break;
-                case '$': finfo |= INFO_J_FLUSH; break;
+                case '-': finfo |= INFO_A_LEFT; break;
+                case '|': finfo |= INFO_A_CENTRE; break;
+                case '$': finfo |= INFO_A_JUSTIFY; break;
                 case '@': finfo |= INFO_ARRAY; break;
                 case '=': finfo |= INFO_COLS; break;
                 case '#': finfo |= INFO_TABLE; break;
@@ -1943,6 +1985,11 @@ static char buff[BUFF_SIZE]; /* The buffer to return the result */
                         if (!fs)
                             ERROR(ERR_CST_REQUIRES_FS);
 
+                        if ((finfo & (INFO_COLS | INFO_TABLE))
+                          == (INFO_COLS | INFO_TABLE)
+                           )
+                            ERROR(ERR_INVALID_FORMAT_STR);
+
                         /* Find the end of the list of columns/tables */
                         temp = &st->csts;
                         while (*temp)
@@ -1963,15 +2010,13 @@ static char buff[BUFF_SIZE]; /* The buffer to return the result */
                             (*temp)->start = st->bpos - st->line_start;
 
                             /* Format the first line from the column */
-                            if (2 == add_column(st, temp))
-                            {
-                                ADD_CHAR('\n');
-                            }
+                            column_stat = add_column(st, temp);
                         }
                         else
                         {
                             /* (finfo & INFO_TABLE) */
                             unsigned int n, len, max;
+                            int tpres;
                             char c, *s, *start;
                             p_uint i;
 
@@ -2015,38 +2060,39 @@ static char buff[BUFF_SIZE]; /* The buffer to return the result */
                              *      max = max length of the lines
                              */
 
-                            if (pres)
+                            tpres = pres;
+                            if (tpres)
                             {
-                                (*temp)->size = fs/pres;
+                                (*temp)->size = fs/tpres;
                             }
                             else
                             {
                                 len = s - start;
                                 if (len > max)
                                     max = len; /* the null terminated word */
-                                pres = fs/(max+2);
+                                tpres = fs/(max+2);
                                   /* at least two separating spaces */
-                                if (!pres)
-                                    pres = 1;
-                                (*temp)->size = fs/pres;
+                                if (!tpres)
+                                    tpres = 1;
+                                (*temp)->size = fs/tpres;
                             }
 
-                            len = n/pres; /* length of average column */
+                            len = n/tpres; /* length of average column */
 
-                            if (n < (unsigned int)pres)
-                                pres = n;
-                            if (len*pres < n)
+                            if (n < (unsigned int)tpres)
+                                tpres = n;
+                            if (len*tpres < n)
                                 len++;
-                            if (len > 1 && n%pres)
-                                pres -= (pres - n%pres)/len;
+                            if (len > 1 && n%tpres)
+                                tpres -= (tpres - n%tpres)/len;
 
-                            (*temp)->d.tab = xalloc(pres*sizeof(char *));
+                            (*temp)->d.tab = xalloc(tpres*sizeof(char *));
                             if (!(*temp)->d.tab)
                                 ERROR(ERR_NOMEM);
-                            (*temp)->nocols = pres; /* heavy sigh */
+                            (*temp)->nocols = tpres; /* heavy sigh */
                             (*temp)->d.tab[0] = TABLE;
 
-                            if (pres == 1)
+                            if (tpres == 1)
                                 goto add_table_now;
 
                             /* Subformat the table, replacing some characters
@@ -2065,7 +2111,7 @@ static char buff[BUFF_SIZE]; /* The buffer to return the result */
                                         SAVE_CHAR(((TABLE)+fs));
                                         TABLE[fs] = '\0';
                                         (*temp)->d.tab[i++] = TABLE+fs+1;
-                                        if (i >= (unsigned int)pres)
+                                        if (i >= (unsigned int)tpres)
                                             goto add_table_now;
                                         n = 0;
                                     }
@@ -2079,7 +2125,7 @@ add_table_now:
                     }
                     else
                     {
-                        Bool flushString;
+                        Bool justifyString;
 
                         /* not column or table */
                         if (pres && pres < slen)
@@ -2088,10 +2134,10 @@ add_table_now:
                         }
 
                         /* Determine whether to print this string
-                         * flushed, or not.
+                         * justified, or not.
                          */
-                        flushString = MY_FALSE;
-                        if (fs && (finfo & INFO_J) == INFO_J_FLUSH)
+                        justifyString = MY_FALSE;
+                        if (fs && (finfo & INFO_A) == INFO_A_JUSTIFY)
                         {
                             /* Flush the string only if it doesn't
                              * end in a NL.
@@ -2106,18 +2152,18 @@ add_table_now:
                               && (unsigned int)slen <= fs
                               && carg->u.string[slen-1] != '\n'
                                )
-                                flushString = MY_TRUE;
+                                justifyString = MY_TRUE;
                         }
 
                         /* not column or table */
 
-                        if (flushString)
+                        if (justifyString)
                         {
-                            add_flush(st, carg->u.string, slen, fs);
+                            add_justified(st, carg->u.string, slen, fs);
                         }
                         else if (fs && fs > (unsigned int)slen)
                         {
-                            add_justified(st, carg->u.string, slen, pad, fs
+                            add_aligned(st, carg->u.string, slen, pad, fs
                                          , finfo);
                         }
                         else
@@ -2205,18 +2251,18 @@ add_table_now:
                              || temp[0] == '+'
                              || temp[0] == '-'
                             )
-                         && (finfo & INFO_J) != INFO_J_LEFT
+                         && (finfo & INFO_A) != INFO_A_LEFT
                            )
                         {
-                            /* Non-left justification and we're printing
+                            /* Non-left alignment and we're printing
                              * with leading zeroes: preserve the sign
                              * character in the right place.
                              */
                             ADD_STRN(temp, 1);
-                            add_justified(st, temp+1, tmpl-1, pad, fs-1, finfo);
+                            add_aligned(st, temp+1, tmpl-1, pad, fs-1, finfo);
                         }
                         else
-                            add_justified(st, temp, tmpl, pad, fs, finfo);
+                            add_aligned(st, temp, tmpl, pad, fs, finfo);
                     }
                     else
                         ADD_STRN(temp, tmpl)
