@@ -99,6 +99,13 @@ static mp_uint num_str_searches = 0;
   /* Number of searches in the string table.
    */
 
+/* TODO: Add these statistics to debug_info() */
+static mp_uint num_found = 0;       /* Number of successful searches */
+static mp_uint num_added = 0;       /* Number of strings added */
+static mp_uint num_deleted = 0;     /* Number of strings deleted */
+static mp_uint num_collisions = 0;  /* Number of hashcollisions when adding */
+static mp_uint num_entries = 0;     /* Number of used cache entries */
+
 /*-------------------------------------------------------------------------*/
 
 /* Hash function, adapted to our table size.
@@ -452,6 +459,7 @@ findstring (const char *s)
                  , scurr, scurr->str, (unsigned long)scurr->ref, scurr->snext);
             fflush(stdout);
 #endif
+            num_found++;
             return curr;
         }
         prev = curr;
@@ -494,6 +502,12 @@ alloc_new_string (const char *str)
     s += SHSTR_OVERHEAD;
 
     /* Copy the string and add it to the string table */
+    if (base_table[h] == NULL)
+        num_entries++;
+    else
+        num_collisions++;
+    num_added++;
+
     strcpy(s, str);
     NEXT(s) = base_table[h];
     base_table[h] = s;
@@ -757,6 +771,10 @@ free_string (char *str)
     s = findstring(str); /* moves it to head of table if found */
 #endif
 
+    num_deleted++;
+    if (NEXT(str) == NULL)
+        num_entries--;
+
 #ifndef KEEP_STRINGS
     base_table[hash_index] = NEXT(str);
 #ifdef CHECK_STRINGS
@@ -803,11 +821,14 @@ add_string_status (strbuf_t *sbuf, Bool verbose)
     net_bytes_distinct_strings
       =   (bytes_distinct_strings & (malloc_size_mask() * sizeof (char *)))
         - num_distinct_strings * SHSTR_OVERHEAD;
-    strbuf_addf(sbuf, "Strings malloced\t\t%8lu %8lu + %lu overhead\n",
+    strbuf_addf(sbuf, "Strings malloced\t\t%8lu %9lu + %lu overhead\n",
                 num_distinct_strings, net_bytes_distinct_strings, overhead_bytes());
 
     if (verbose)
     {
+        mp_uint num_x_searches;
+
+        num_x_searches = num_str_searches ? num_str_searches : 1;
         stralloc_allocd_bytes &= malloc_size_mask();
         net_allocd_bytes =   (stralloc_allocd_bytes * sizeof(char*))
                            - stralloc_allocd_strings * SHSTR_OVERHEAD;
@@ -816,9 +837,20 @@ add_string_status (strbuf_t *sbuf, Bool verbose)
         strbuf_addf(sbuf, "Space actually required/total string bytes %lu%%\n",
                     (net_bytes_distinct_strings + overhead_bytes())*100L /
                             net_allocd_bytes );
-        strbuf_addf(sbuf, "Searches: %d    Average search length:%7.3f\n",
-                    num_str_searches
-                    , (float)search_len / (float)num_str_searches);
+        strbuf_addf(sbuf, "Searches: %d - Found: %lu (%.1f%%) - Average search length:%7.3f\n"
+                   , num_str_searches
+                   , num_found, 100.0 * (float)num_found / (float) num_x_searches
+                   , (float)search_len / (float)num_x_searches);
+        strbuf_addf(sbuf, "Number of chains: %lu (%.1f%%)\n"
+                        , num_entries, 100.0 * (float)num_entries/(float)HTABLE_SIZE
+                   );
+        strbuf_addf(sbuf, "Strings added: %lu - Deleted: %lu - Coll: %lu (%.1f%% added/%.1f%% chains)\n"
+                        , num_added
+                        , num_deleted
+                        , num_collisions
+                        , 100.0 * (float)num_collisions/(float)num_added
+                        , 100.0 * (float)num_collisions/(float)num_entries
+                   );
     }
 
     return net_bytes_distinct_strings + overhead_bytes();
@@ -860,6 +892,12 @@ string_dinfo_status (svalue_t *svp, int value)
 
     ST_NUMBER(DID_ST_STR_SEARCHES, num_str_searches);
     ST_NUMBER(DID_ST_STR_SEARCH_LEN, search_len);
+    ST_NUMBER(DID_ST_STR_FOUND, num_found);
+
+    ST_NUMBER(DID_ST_STR_ENTRIES, num_entries);
+    ST_NUMBER(DID_ST_STR_ADDED, num_added);
+    ST_NUMBER(DID_ST_STR_DELETED, num_deleted);
+    ST_NUMBER(DID_ST_STR_COLLISIONS, num_collisions);
 
 #undef ST_NUMBER
 } /* string_dinfo_status() */
