@@ -22,10 +22,10 @@ struct strbuf_s
 
 extern void strbuf_zero(strbuf_t * buf);
 extern void strbuf_free(strbuf_t * buf);
-extern void strbuf_add(strbuf_t *buf, char * text);
-extern void strbuf_addn(strbuf_t *buf, char * text, size_t len);
+extern void strbuf_add(strbuf_t *buf, const char * text);
+extern void strbuf_addn(strbuf_t *buf, const char * text, size_t len);
 extern void strbuf_addc(strbuf_t *buf, char ch);
-extern void strbuf_addf(strbuf_t *buf, char *format, ...);
+extern void strbuf_addf(strbuf_t *buf, const char *format, ...);
 extern void strbuf_send(strbuf_t *buf);
 extern void strbuf_store(strbuf_t *buf, svalue_t *svp);
 extern char * intersect_strings (char * left, char * right, Bool bSubtract);
