@@ -100,6 +100,7 @@
 #include "main.h"
 #include "mapping.h"
 #include "object.h"
+#include "simulate.h"
 #include "stdstrings.h"
 #include "stralloc.h"
 #include "svalue.h"
@@ -299,6 +300,10 @@ get_wiz_name (char *file)
     svalue_t *ret;
     static char buff[50];
 
+    /* Don't call the master if it isn't loaded! */
+    if (!master_ob)
+        return NULL;
+
     push_volatile_string(file);
     ret = apply_master_ob(STR_GET_WNAME, 1);
     if (ret == 0 || ret->type != T_STRING)
